/* 
 * TextOutput 
 * 
 * System to display a list of text output in a single sprite batch output for XNA.
 * 
 * Author: Kurt Wesner, Bob Bradley
 * Place: University of Tennessee at Martin
 * Date:  July 02 2007
 * 
 */
#region Using Statements
using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using System.Collections.Generic;
#endregion

namespace _297Final
{



    /// <summary>
    /// This XNA class allows multiple lines of text to be output to the screen. 
    /// The text is stored in a list and is output in a single SpriteBatch.
    /// This is an XNA:DrawableGameComponent that will draw itself.
    /// All that has to be done is add this component to your main game.
    /// 
    /// <code>
    /// 
    ///    TextOutput textOutput;
    /// 
    ///    public Game1()
    ///     {
    ///        //put the following line in your constructor
    ///        this.Components.Add(textOutput = new TextOutput(this));
    ///        
    ///    }
    /// 
    ///        //use the following to output text
    ///        textOutput.WriteAt(50, 50, "Hello World!");
    /// 
    /// </code>
    /// 
    /// 
    /// </summary> 
    public class TextOutput : DrawableGameComponent
    {

        #region Nested Types
        /// <summary>
        ///  Internal Structure used to hold text and attributes to be drawn.
        /// </summary>
        struct textNode
        {
            public int X, Y;
            public string Text;
            public Color FontColor;
            public float Rotation;
        }

        #endregion

        #region Fields and Properties

        /// <summary>
        /// Stores managed Objects for the component.
        /// </summary>
        private ContentManager m_ContentManager;

        /// <summary>
        /// Pointer to base game.
        /// </summary>
        private Game m_Game;

        /// <summary>
        /// List of strings and attributes to be output.
        /// </summary>
        private List<textNode> m_List = new List<textNode>();

        /// <summary>
        ///  Batch used to render font.
        /// </summary>
        private SpriteBatch m_SpriteBatch;

        /// <summary>
        /// Font used in Batch.
        /// </summary>
        private SpriteFont m_SpriteFont;


        /// <summary>
        /// Rotation to render text.
        /// </summary>
        public float FontRotation
        {
            get { return m_FontRotation; }
            set { m_FontRotation = value; }
        }
        private float m_FontRotation = 0;

        /// <summary>
        /// Color to render text.
        /// </summary>
        public Color FontColor
        {
            get { return m_FontColor; }
            set { m_FontColor = value; }
        }
        private Color m_FontColor = Color.AntiqueWhite;

        /// <summary>
        /// Enables or disables drawing of all text.
        /// </summary>
        public bool Enable
        {
            get { return m_Enabled; }
            set { m_Enabled = value; }
        }
        private bool m_Enabled = true;

        #endregion

        #region Methods
        /// <summary>
        /// Default Constructor
        /// </summary>
        /// <param name="game">Pointer to base game.</param>
        public TextOutput(Game game)
            : base(game)
        {
            m_Game = game;

        }

        /// <summary>
        ///  Adds text to be displayed at position (x,y).
        /// </summary>
        /// <param name="x">x location on screen</param>
        /// <param name="y">y location on screen</param>
        /// <param name="s">Text to be displayed</param>
        public void WriteAt(int x, int y, string s)
        {
            WriteAt(x, y, s, FontColor);
        }

        /// <summary>
        ///  Adds text to be displayed at position (x,y).
        /// </summary>
        /// <param name="x">x location on screen</param>
        /// <param name="y">y location on screen</param>
        /// <param name="s">Text to be displayed</param>
        /// <param name="c">Color of Text</param>
        public void WriteAt(int x, int y, string s, Color c)
        {
            //create a new textNode 
            textNode n = new textNode();
            //populate x pos
            n.X = x;
            //populate y pos
            n.Y = y;
            //populate string 
            n.Text = s;
            //set color
            n.FontColor = c;
            //set rotation
            n.Rotation = m_FontRotation;
            //add to the batch list
            m_List.Add(n);
        }

        /// <summary>
        /// Enables rendering of Text.
        /// </summary>
        public void Show()
        {
            m_Enabled = true;
        }

        /// <summary>
        /// Disables rendering of Text.
        /// </summary>
        public void Hide()
        {
            m_Enabled = false;
        }

        /// <summary>
        /// Writes a string at the current Mouse location.
        /// </summary>
        /// <param name="s">Text to be displayed.</param>
        public void writeAtMouse(string s)
        {
            writeAtMouse(s, m_FontColor);
        }

        /// <summary>
        /// Writes a string at the current Mouse Location.
        /// </summary>
        /// <param name="s">Text to be Displayed.</param>
        /// <param name="c">Color of Text to be Displayed.</param>
        public void writeAtMouse(string s, Color c)
        {
#if !XBOX360
            MouseState currentState = Mouse.GetState();
            WriteAt(currentState.X + 16, currentState.Y, s, c);
#endif
        }
        /// <summary>
        /// Draws all text. Called automatically by the component system.
        /// </summary>
        /// <param name="gameTime"></param>
        public override void Draw(GameTime gameTime)
        {
            base.Draw(gameTime);
            if (m_Enabled)//if we want text rendered 
            {
                m_SpriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.FrontToBack, SaveStateMode.SaveState);
                //foregBatch.Begin();
                foreach (textNode n in m_List)
                {
                    m_SpriteBatch.DrawString(m_SpriteFont, n.Text, new Vector2(n.X, n.Y), n.FontColor,
                        n.Rotation, new Vector2(0, 0), 1.0f, SpriteEffects.None, 0.5f);
                }
                m_SpriteBatch.End();
                m_List.Clear();
            }//end if (m_Enabled)
        }

        /// <summary>
        /// Load function for the textoutput. this hooks the graphics and content 
        /// into this function so that it can be batched for output.
        /// </summary>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {

            base.LoadGraphicsContent(loadAllContent);
            m_SpriteBatch = new SpriteBatch(this.GraphicsDevice);
            m_ContentManager = new ContentManager(m_Game.Services);
            m_SpriteFont = m_ContentManager.Load<SpriteFont>("Content/Arial");


        }

        #endregion

    }
}
